#!/bin/bash

MASK_DEFAULT="255.255.255.0"

dec2hex(){
	printf "0x%x" $1
}

mask_to_cidr() {
    local mask=$1
    local octet binary_octet total_binary cidr
    IFS='.' read -r -a octets <<< "$mask"
    total_binary=""

    for octet in "${octets[@]}"; do
        # Convert base 10 to base 2 and fill to 8bit
        binary_octet=$(bc <<< "obase=2; $octet" | tr -d '\\\n')
        binary_octet=$(printf "%08d" "$binary_octet")
        total_binary+="$binary_octet"
    done

    # Caculate number of consecutive 1s
    cidr=$(grep -o '^1*' <<< "$total_binary" | wc -c)
    echo $((cidr - 1))  # Substract final '\n'
}

netlist="eth0 eth1"
for net in $netlist; do
	if [ -e /sys/class/net/$net ]; then
		echo 1 > /sys/class/net/$net/threaded
		macaddr=`hrut_mac g $net`
		if [ "$macaddr" == "ff:ff:ff:ff:ff:ff" ];then
			hrut_mac c $net
		fi

		macaddr=`hrut_mac g $net`
		tddr=$(hrut_mac g eth0 | awk 'NR==3 {print $0}')
		if [ "$macaddr" == "00:00:00:00:00:00" ];then
			addr=$(dd if=/dev/urandom bs=1024 count=1 2>/dev/null|md5sum|sed 's/^\(..\)\(..\)\(..\)\(..\)\(..\).*$/\1:\2:\3:\4:\5/')
			addr_0=$(dd if=/dev/urandom bs=1024 count=1 2>/dev/null|md5sum|sed 's/^\(..\).*$/\1/')
			addr_0=`printf '%d\n' 0x$addr_0`
			addr_0=$((addr_0|0x2))
			addr_0=$((addr_0&0xfe))
			addr_0=`printf '%x\n' $addr_0`
			macaddr="$addr_0:$addr"
			hrut_mac  s $net $macaddr
		fi
		echo $macaddr

		# TODO: Distinguish if the system is equipped with netplan
		netplan set ethernets.$net.macaddress="$macaddr"
		hrut_ipfull g $net
		if [ -f /tmp/ip_mask_$net ];then
			MASK=`cat /tmp/ip_mask_$net`
		else
			MASK="$MASK_DEFAULT"
		fi

		if [ -f /tmp/ip_ip_$net ];then
			IP=`cat /tmp/ip_ip_$net`
			ip_with_cidr="${IP}/$(mask_to_cidr $MASK)"
			# Disable dhcp
			netplan set ehternets.$net.dhcp4=no
			netplan set ehternets.$net.dhcp6=no
			netplan set ethernets.$net.addresses="[$ip_with_cidr]"
		fi

		netplan apply
		# TODO: Distinguish between Desktop and server version
		#       to make sure nmcli exists
		nmcli c modify netplan-$net connection.autoconnect yes
		if [ "$(netplan get ehternets.$net.dhcp4)" != "true" ];then
			nmcli c up netplan-$net
		fi

		if [ -f /tmp/ip_gw_$net ];then
			GATEWAY=`cat /tmp/ip_gw_$net`
			route add default gw $GATEWAY
		fi
	fi
done
exit 0
