#!/bin/bash

CURD=`pwd`


FPS=30
PIPE=
LOGF=
LOGE=
SOUT=
SERR=
SEPT=1
FPS_PARSE=y
FPS_WARN=2
WARN_ASOK=
WARN_ASERR=
RUNT=10000000


# script usage.
usage()
{
	cat <<EOF
`basename $0` [logfile] [-f#] [-w#] [-n] [-h]
  logfile - path of log to save, default stdout
  -f#     - fps target list, default ${FPS}
  -w#     - fps warn diff, default ${FPS_WARN}
  -n      - no fps parse and timestamp, only watch
  -h      - show this help doc
EOF
}

# get current time
CURT()
{
	date +%y%m%d_%T.%6N
}

# clean and init logfile.
LOGINIT()
{
	CUR=$(CURT)
	if [ "${LOGF}" != "stdout" ]; then
		echo -e "[${CUR}] -- I: ${LOGF}" > ${LOGF}
	fi
	if [ "${LOGE}" != "stderr" ]; then
		echo -e "[${CUR}] -- E: ${LOGE}" > ${LOGE}
	fi
}

# log out to LOGF.
LOGI()
{
	CUR=$(CURT)
	if [ "${LOGF}" = "stdout" ]; then
		echo -e "[${CUR}] -- $*"
	else
		echo -e "[${CUR}] -- $*" >> ${LOGF}
		if [ ! -z "${SOUT}" ]; then
			echo -e "[${CUR}] -- $*"
		fi
	fi
}

# log out to LOGE&LOGF.
LOGX()
{
	CUR=$(CURT)
	if [ "${LOGE}" = "stderr" ]; then
		echo -e "[${CUR}] ** $*" 1>&2
	else
		echo -e "[${CUR}] ** $*" >> ${LOGF}
		echo -e "[${CUR}] ** $*" >> ${LOGE}
		if [ ! -z "${SERR}" ]; then
			echo -e "[${CUR}] ** $*" 1>&2
		fi
	fi
}

# log out warn.
LOGW()
{
	if [ -z "${WARN_ASERR}" ]; then
		CUR=$(CURT)
		if [ "${LOGF}" = "stdout" ]; then
			echo -e "[${CUR}] ## $*"
		else
			echo -e "[${CUR}] ## $*" >> ${LOGF}
			if [ ! -z "${SOUT}" ]; then
				echo -e "[${CUR}] ## $*"
			fi
		fi
	else
		LOGX $*
	fi
}

# log out raw to LOGF.
LOGR()
{
	if [ "${LOGE}" = "stderr" ]; then
		echo -e "$*" 1>&2
	else
		echo -e "$*" >> ${LOGF}
		if [ ! -z "${SERR}" ]; then
			echo -e "$*" 1>&2
		fi
	fi
}

# args.
for arg in $*; do
	if [ "${arg}" == "-h"  ]; then
		usage
		exit 0
	elif [ "${arg:0:2}" == "-f"  ]; then
		FPS="${arg:2}"
	# elif [ "${arg:0:2}" == "-t"  ]; then
	# 	SEPT="${arg:2}"
	elif [ "${arg:0:2}" == "-w"  ]; then
		if [ -z "${arg:2}" ]; then
			WARN_ASOK=y
		elif [ "${arg:2:1}" == "-" ]; then
			WARN_ASOK=y
			if [ ! -z "${arg:3}" ]; then
				FPS_WARN="${arg:3}"
			fi
		else
			FPS_WARN="${arg:2}"
		fi
	# elif [ "${arg}" == "-W"  ]; then
	# 	WARN_ASERR=y
	elif [ "${arg}" == "-n"  ]; then
		FPS_PARSE=
	# elif [ "${arg}" == "-s"  ]; then
	# 	SERR=y
	# elif [ "${arg}" == "-S"  ]; then
	# 	SOUT=y
	# 	SERR=y
	elif [ "${arg//./}" != "${arg}" ]; then
		if [ -z "${LOGF}" ]; then
			LOGF=${arg}
		elif [ -z "${LOGE}" ]; then
			LOGE=${arg}
		else
			echo "** logfile too more: ${arg}"
			exit 1
		fi
	elif [ -z "${arg//[0-9]/}" ]; then
		RUNT=${arg}
	elif [ "${arg:0:1}" == "-"  ]; then
		echo "** not support ${arg}"
		exit 1
	else
		echo "** not support ${arg}"
		exit 1
	fi
done


# get logfiel.
if [ -z "${LOGF}" ]; then
	LOGF=stdout
elif [ "${LOGF:0:1}" != "/" ]; then
	LOGF=${CURD}/${LOGF}
fi
if [ -z "${LOGE}" ]; then
	if [ "${LOGF}" = "stdout" ]; then
		LOGE=stderr
	else
		LOGE=`echo ${LOGF} |sed -e 's/\(.*\)\.\(.*\)/\1_error.\2/'`
	fi
elif [ "${LOGE:0:1}" != "/" ]; then
	LOGE=${CURD}/${LOGE}
fi

# fps pipe.
# $1 - pipe id.
# $? - get fps from FPS.
fps_pipe()
{

	if [ "${FPS//,/}" = "${FPS}" ]; then
		echo "${FPS}"
	else
		get=`echo ${FPS} |awk -F ',' -v n=$(($1+1)) '{print $n}'`
		if [ -z "${get}" ]; then
			get=`echo ${FPS} |awk -F ',' '{print $1}'`
		fi
		echo "${get}"
	fi
}

# fps watch.
pipe_init_mask=0
fps_do()
{
	flow_fps_info=$(cat /sys/devices/virtual/vps/flow/fps)
	ret=0
	pipe_has=0
	while IFS= read -r line; do
		if [[ $line == *Flow* ]]; then
			p=$(echo $line | grep -Eo 'Flow[0-9]+' | grep -Eo '[0-9]+')
			LOGI "PIPE (KERNEL Flow) ID: $p"
		elif [[ $line == *ich0* ]]; then
			ln=1
			if [ -z "${FPS_PARSE}" ]; then
				LOGR "${line}"

				ln=$(($ln + 1))
				line=$(echo "${fpsx}" |sed -n "${ln}p")
				continue
			fi
			# parse.
			dev=$(echo $line |grep -Eo '.+[0-9]+ ctx +[0-9]+')
			ifps=$(echo $line | grep -Eo 'ich0\ +[0-9]+'| awk -F ' ' '{print $2}')
			if [[ $line == *vin* ]]; then
				ofps0=$(echo $line | grep -Eo 'och0\ +[0-9]+'|awk -F ' ' '{print $2}')
				ofps1=$(echo $line | grep -Eo 'och1\ +[0-9]+'|awk -F ' ' '{print $2}')
				ofps3=$(echo $line | grep -Eo 'och3\ +[0-9]+'|awk -F ' ' '{print $2}')
				ofps4=$(echo $line | grep -Eo 'och4\ +[0-9]+'|awk -F ' ' '{print $2}')
				ofps=$(echo "$ofps0 $ofps1 $ofps3 $ofps4" | tr ' ' '\n' | sort -n | awk '$1!=0 {print; exit}')
			else
				ofps=$(echo $line | grep -Eo 'och0\ +[0-9]+'|awk -F ' ' '{print $2}')
			fi

			pipe_has=$((${pipe_has} | (0x1 << ${p})))
			if [ "${dev:0:3}" = "sth" ]; then
				fmt="\t"
			elif [ "${dev:0:3}" = "vin" ]; then
				fmt="\t"
			else
				fmt="\t\t\t\t\t"
			fi
			fps_x=$(fps_pipe $p)
			fps_warn_min=$((${fps_x} - ${FPS_WARN}))
			fps_warn_max=$((${fps_x} + ${FPS_WARN}))

			if [ ${ifps} -eq ${fps_x} ] && [ ${ofps} -eq ${fps_x} ]; then
				# fps match.
				if [ $(((${pipe_init_mask} >> ${p}) & 0x1)) -eq 0 ]; then
					# not inited, as inited.
					pipe_init_mask=$((${pipe_init_mask} | (0x1 << ${p})))
					LOGI "${line}${fmt}| pipi${p} ready ${fps_x}"
					if [ "${LOGF}" != "stdout" ]; then
						echo -e "[$(CURT)] -- ${line}${fmt}| pipi${p} ready ${fps_x}"
					fi
				else
					# inited, as ok.
					LOGI "${line}${fmt}| ok"
				fi
			elif [ ${ifps} -ge ${fps_warn_min} ] && [ ${ifps} -le ${fps_warn_max} ] && [ ${ofps} -ge ${fps_warn_min} ] && [ ${ofps} -le ${fps_warn_max} ]; then
				# fps warn.
				if [ $(((${pipe_init_mask} >> ${p}) & 0x1)) -eq 0 ]; then
					# not inited, as initing.
					LOGI "${line}${fmt}| pipe${p} wait ${fps_x} ..."
				else
					# inited, as warn.
					if [ -z "${WARN_ASOK}" ]; then
						LOGW "${line}${fmt}| warn: ${ifps}/${ofps}"
						if [ $ret -eq 0 ]; then
							ret=1
						fi
					else
						# warn as ok.
						LOGW "${line}${fmt}| ok"
					fi
				fi
			else
				# fps error.
				if [ $(((${pipe_init_mask} >> ${p}) & 0x1)) -eq 0 ]; then
					# not inited, as initing.
					LOGI "${line}${fmt}| pipe${p} wait ${fps_x} ..."
				else
					# inited, as error.
					LOGX "${line}${fmt}| error: ${ifps}/${ofps} (pipe${p}:${dev})"
					ret=2
				fi
			fi
			ln=$(($ln + 1))
		fi
	done <<< "$flow_fps_info"

	if [ ! -z "${FPS_PARSE}" ]; then
		if [ ${pipe_init_mask} -ne ${pipe_has} ]; then
			pipe_mask=${pipe_init_mask}
			p=0
			while [ ${pipe_mask} -ne 0 ]; do
				if [ $(((${pipe_mask} >> ${p}) & 0x1)) -ne 0 ] && [ $(((${pipe_has} >> ${p}) & 0x1)) -eq 0 ] ; then
					LOGX "\t\t\t\t\t\t\t| error: no stream (pipe${p})"
					ret=2
				fi
				pipe_mask=$((${pipe_mask} & ~(0x1 << ${p})))
				p=$(($p + 1))
			done
		fi
	fi
	return $ret
}

# start.
if [ -z "${FPS_PARSE}" ]; then
	echo "start watch ${SEPT}s "
	echo "fps=${FPS}(+-${FPS_WARN}${WARN_ASOK}) logf=${LOGF} loge=${LOGE}"
else
	echo "[$(CURT)] -- start watch ${SEPT}s "
	echo "[$(CURT)] -- fps=${FPS}(+-${FPS_WARN}${WARN_ASOK}) logf=${LOGF} loge=${LOGE}"
fi
LOGINIT

n=1
w=0
e=0
sep_us=$((${SEPT} * 1000000))
while [ $n -le ${RUNT} ]; do
	fps_do
	ret=$?
	if [ $ret -eq 2 ]; then
		e=$(($e + 1))
		LOGX "--------------------------------------------------- $n (error=${e})"
	elif [ $ret -eq 1 ]; then
		w=$(($w + 1))
		LOGW "--------------------------------------------------- $n (warn=${w})"
	elif [ -z "${FPS_PARSE}" ]; then
		LOGR "--------------------------------------------------- $n"
	else
		LOGI "--------------------------------------------------- $n (e${e} w${w})"
	fi
	n=$(($n + 1))
	usleep ${sep_us}
done
