#!/bin/bash

key_is_random() {
        [ "$1" = "/dev/urandom" -o "$1" = "/dev/hw_random" \
        -o "$1" = "/dev/random" ]
}

init_crypto() {
        local have_random dst src key opt mode owner params makeswap skip arg opt
        local param value rc ret mke2fs mdir mount_point item mkext4

        while read -r line; do
                dst=$(echo "$line" | awk '{print $1}')
                src=$(echo "$line" | awk '{print $2}')
                key=$(echo "$line" | awk '{print $3}')
                opt=$(echo "$line" | awk '{print $4}')
                [ -z "$dst" -o "${dst#\#}" != "$dst" ] && continue
                [ -b "/dev/mapper/$dst" ] && continue;
                if [ "$have_random" = 0 ] && key_is_random "$key"; then
                        continue
                fi
                if [ -n "$key" -a "x$key" != "xnone" -a "x$key" != "x--key-hobot" ]; then
                        if test -e "$key" ; then
                        mode=$(ls -l "$key" | cut -c 5-10)
                        owner=$(ls -l $key | awk '{ print $3 }')
                        if [ "$mode" != "------" ] && ! key_is_random "$key"; then
                                echo "INSECURE MODE FOR $key"
                        fi
                        if [ "$owner" != root ]; then
                                echo "INSECURE OWNER FOR $key"
                        fi
                        else
                        echo "Key file for $dst not found"
                        fi
                        key="-d $key"
                else
                        key="--key-hobot"
                fi
                params=""
                makeswap=""
                mke2fs=""
                skip=""
                # Parse the src field for UUID= and convert to real device names
                if [ "${src%%=*}" == "UUID" ]; then
                        src=$(/sbin/blkid -t "$src" -l -o device)
                elif [ "${src/^\/dev\/disk\/by-uuid\/}" != "$src" ]; then
                        src=$(__readlink $src)
                fi
                # Is it a block device?
                [ -b "$src" ] || continue
                # Is it already a device mapper slave? (this is gross)
                devesc=${src##/dev/}
                devesc=${devesc//\//!}
                for d in /sys/block/dm-*/slaves ; do
                        [ -e $d/$devesc ] && continue 2
                done
                while [ -n "$opt" ]; do
                        arg=${opt%%,*}
                        opt=${opt##$arg}
                        opt=${opt##,}
                        param=${arg%%=*}
                        value=${arg##$param=}

                        case "$param" in
                        cipher)
                        echo "CRYPTSETUP: cipher value: $value"
                        params="$params -c $value"
                        if [ -z "$value" ]; then
                                echo $"$dst: no value for cipher option, skipping"
                                skip="yes"
                        fi
                        ;;
                        size)
                        params="$params -s $value"
                        if [ -z "$value" ]; then
                                echo  $"$dst: no value for size option, skipping"
                                skip="yes"
                        fi
                        ;;
                        hash)
                        params="$params -h $value"
                        if [ -z "$value" ]; then
                                echo $"$dst: no value for hash option, skipping\n"
                                skip="yes"
                        fi
                        ;;
                        verify)
                        params="$params -y"
                        ;;
                        swap)
                        makeswap=yes
                        ;;
                        tmp)
                        mke2fs=yes
                        ;;
                        ext4)
                        mkext4=yes
                        esac
                done
                if [ "$skip" = "yes" ]; then
                        ret=1
                        continue
                fi

                echo "CRYPTSETUP: $src"
                if cryptsetup isLuks "$src" 2>/dev/null; then
                        if key_is_random "$key"; then
                        echo "$dst: LUKS requires non-random key, skipping\n"
                        ret=1
                        continue
                        fi
                        echo "CRYPTSETUP: $params"
                        cryptsetup ${key:+$key} $params -v luksOpen "$src" "$dst" <&1
                        rc=$?
                        if [ $rc -ne 0 ]; then
                        echo "ERR: fde luksOpen $src fail"
                        return $rc;
                        fi
                else
                        echo "INFO: fde luksFormat $src starting"
                        #root device erases. issue: when non-fde to fde to non-fde, fsck will repair root device.
                        if [ -n "$(blkid $src | grep -w ext4)" ]; then
                        blkdiscard $src
                        if [ $? -ne 0 ]; then
                                echo "WARN: fde blkdiscard $src fail"
                        fi
                        fi
                        echo "CRYPTSETUP: key:$key src:$src dst:$dst"
                        #cryptsetup $params ${key:+$key} -v create "$dst" "$src" <&1 2>/dev/null
                        #source /etc/init.d/forceencrypt.sh $src $dst $mount_point "$key" "$params"
                        #umount $src
                        cryptsetup $params --hash=sha512 -q luksFormat $src $key
                        rc=$?
                        if [ $rc -ne 0 ]; then
                        echo "ERR: fde luksFormat $src fail"
                        return $rc
                        else
                        cryptsetup $params luksOpen $src $dst $key
                        fi

                        rc=$?
                        if [ $rc -ne 0 ];then
                        echo "ERR: fde luksOpen $src fail"
                        return $rc;
                        fi
                fi
                echo "CRYPTSETUP: $dst"
                if [ -b "/dev/mapper/$dst" ]; then
                        if [ "$mkext4" = "yes" ]; then
                                #mount ext4 partition
                                mkdir -p /$dst
                                echo "mount -t ext4 /dev/mapper/$dst /$dst"
                                fsck.ext4 -yf /dev/mapper/$dst
                                mount -t ext4 /dev/mapper/$dst /$dst
                                if [ $?  != 0 ];then
                                        mkfs.ext4 -O 64bit -F /dev/mapper/$dst
                                        fsck.ext4 -yf /dev/mapper/$dst
                                        mount -t ext4 /dev/mapper/$dst /$dst
                                fi
                        elif [ "$makeswap" = "yes" ]; then
                                mkswap "/dev/mapper/$dst" 2>/dev/null >/dev/null
                        fi
                fi
        done < /etc/hb-crypttab
}

if [ -f /etc/hb-crypttab ]; then
        init_crypto
fi