#!/bin/bash
#

load_and_configure_gpu_modules()
{
	modprobe mali_gpu_power
	modprobe mali_gpu_system
	modprobe mali_gpu_assign ptm_config='A:S0:P0:W0'
	modprobe mali_gpu_partition_config
	modprobe mali_gpu_partition_control
	modprobe mali_arbiter
	modprobe mali_gpu_resource_group
	echo "0x1" > /sys/bus/platform/devices/260a0000.gpu_resource_group/arbiter/partitions/partition0/active_slices
	echo "0x1" > /sys/bus/platform/devices/260a0000.gpu_resource_group/arbiter/partitions/partition0/assigned_access_windows
	modprobe mali_gpu_aw
	modprobe mali_kbase
}

load_and_configure_vdsp_modules()
{
	modprobe hobot_vdsp
}

load_modules()
{
	modprobe hobot_watchdog
	modprobe i2c-mux
	modprobe i2c-dev
	modprobe regmap-i2c
	modprobe dwc3-of-simple &
	modprobe bpu_hw_io_super &
	modprobe hobot_videosys &
	modprobe hobot_vpu &
	modprobe hobot_jpu &
	modprobe hobot_eth_super &
	modprobe hobot_mdmac &

	modprobe hobot_vio_common
	modprobe hobot_camsys &
	modprobe hobot_pym_jplus
	modprobe hobot_vin_vnode &
	modprobe hobot_cim
	modprobe hobot_mipiphy
	modprobe hobot_mipicsi &
	modprobe hobot_mipidbg &
	modprobe hobot_vin_vcon &
	modprobe hobot_sensor &
	modprobe hobot_deserial &
	modprobe hobot_lpwm &
	modprobe hobot_isp &
	modprobe hobot_ynr
	modprobe hobot_stitch
	modprobe hobot_gdc
	modprobe spidev &
	modprobe hobot-spi &
	modprobe mtd &
	grep -q hobotboot.secureboot=1 /proc/cmdline && modprobe hobot_vspi &
	grep -q hobotboot.secureboot=1 /proc/cmdline || modprobe hobot_vspi_debug &
	modprobe ddr_perf_monitor &
	modprobe hobot_ide_wrapper
	modprobe hobot_idu_drm
	modprobe hobot_idu_vnode
	load_and_configure_gpu_modules
	load_and_configure_vdsp_modules &
	modprobe security_drv &
	modprobe boardinfo &
	modprobe arm_dsu_pmu &
	modprobe qga_msgfwd &
	modprobe super_suspend_mode &
	# TODO: Remove leds-gpio modprobe after uevent properly trigger udev modprobe
	modprobe leds-gpio &
	modprobe ledtrig_activity &
}

# config in release version
grep release /etc/version -wq && {
	echo 1 > /proc/sys/kernel/dmesg_restrict &
	echo 2 > /proc/sys/kernel/unprivileged_bpf_disabled &
	echo 2 > /proc/sys/net/core/bpf_jit_harden &
	echo 2 > /proc/sys/kernel/randomize_va_space &
	wait
}

echo "load_ko_begin"
load_modules
wait

boardid_sys_path="/sys/class/boardinfo/adc_boardid"
if [ -f "$boardid_sys_path" ]; then
	boardid="$(cat $boardid_sys_path)"
	if [[ "$boardid" =~ ^0x(64|65|6A|6B)[0678][0456]$ ]];then
		case ${boardid} in
			*"0")
				# Check if TPIC2810 exists, if so, manual reset USB controller
				if [ -f /sys/class/i2c-adapter/i2c-2/2-0060/name ] &&
				   [ "$(cat /sys/class/i2c-adapter/i2c-2/2-0060/name)" = "tpic2810" ];then
					/usr/bin/pcie-usb-reset.sh
				fi
				/usr/bin/start-pcie.sh &
			;;
			*"6") ;&
			*"5") ;&
			*"4")
				modprobe hobot-pcie-rc
				# check & update asm3042 firmware
				/usr/bin/update-asm3042-firmware.sh
				;;
			*)
				;;
		esac
	fi
fi

echo "load_ko_end"

# Triggle all cpu backtrace on hung-task detection
echo 1 > /proc/sys/kernel/hung_task_all_cpu_backtrace &
# Triggle all cpu backtrace on softlockup detection
echo 1 > /proc/sys/kernel/softlockup_all_cpu_backtrace &
# Triggle all cpu backtrace on Oops
echo 1 > /proc/sys/kernel/oops_all_cpu_backtrace &
# Triggle panic on RCU stall
echo 1 > /proc/sys/kernel/panic_on_rcu_stall &
wait
