#!/bin/bash

function output() {
    echo "mount: $1" > /dev/kmsg &
	echo "mount: $1"
}

function link_mtd() {
	modprobe mtd
	grep -q hobotboot.secureboot=1 /proc/cmdline && modprobe hobot_vspi
	grep -q hobotboot.secureboot=1 /proc/cmdline || modprobe hobot_vspi_debug
	/etc/init.d/by-name.sh mtd
}

function check_ext4_partition()
{
	local partline="$1"
	local device_path=$(echo "${partline}" | awk '{print $1}')
	local part_point=$(echo "${partline}" | awk '{print $2}')
	local read_only_part=$(echo "${partline}" | grep -w "ro")

	if [ ! -e "${device_path}" ]; then
		output "ERR: ${device_path} is not exists"
		return
	fi

	mkdir -p "${part_point}"

	if [ -n "${read_only_part}" ]; then
		# for readonly part, if use fsck.ext4 -yf, will affect ota diff update
		fsck.ext4 -nf ${device_path}
		local ret=$?
		output "INFO: ${device_path} nf fsck result: ${ret}"
		if [ ${ret} -ne 0 ]; then
			fsck.ext4 -yf ${device_path}
			echo "mount -t ext4 -o ro ${device_path} ${part_point}"
			if [ -z "$(mount | grep ${device_path})" ]; then
				mount -t ext4 -o ro ${device_path} ${part_point}
				if [ $? -ne 0 ]; then
					local part_name=${device_path##*/}
					part_name=${part_name%_cur*}
					mkfs.ext4 -O 64bit -L "${part_name}" -F "${device_path}"
					sync
				fi
				umount ${part_point}
			fi
		fi
	else
		fsck.ext4 -yf ${device_path}
		local ret=$?
		output "INFO: ${device_path} yf fsck result: ${ret}"
		if [ ${ret} -ne 0 ]; then
			if [ -z "$(mount | grep ${device_path})" ]; then
				echo "mount -t ext4 -o ro ${device_path} ${part_point}"
				mount -t ext4 -o ro ${device_path} ${part_point}
				if [ $? -ne 0 ]; then
					local part_name=${device_path##*/}
					part_name=${part_name%_cur*}
					mkfs.ext4 -O 64bit -L "${part_name}" -F "${device_path}"
					sync
				fi
				umount ${part_point}
			fi
		fi
	fi
}

link_mtd &
/etc/init.d/by-name.sh

cat /sys/class/boardinfo/bootdevice_name | grep -q scsi || {
	##training for eMMC
	echo "start eMMC training..."
	echo 999 > /sys/kernel/debug/mmc0/phy/training
}

IFSOLD=$IFS
IFS=$'\n'
for line in $(awk '/ ext4 /{print $0}' /etc/hb-fstab); do
	check_ext4_partition "${line}" &
done
wait
IFS=$IFSOLD

mount -a --fstab /etc/hb-fstab
chcon -R u:object_r:user_data_file:s0 /userdata

if [ ! -d "/userdata/data" ]; then
	mkdir -p /userdata/data
fi
# ln -s /userdata/data /
