#!/bin/bash

RED='\033[0;31m'
NC='\033[0m'

function help() {
    echo "USAGE:"
    echo "  $0 wb 4/8/12/...   config writebootster size 4/8/12/... GB"
    echo "  $0 lun             config lun, lun0: main partition, lun1: 32MB boot partition, lun2: 32MB boot partition"
    echo -en "${RED}Attention: need to re-write the image after the configuration is complete!!!${NC}\n\n"
    exit 1
}

function tool_check() {
    which ufs-utils >/dev/null || {
        echo "ufs-utils not found, please install ufs-utils"
        exit 1
    }
}

function ufs_wb_config() {
    echo "config writebooster start..."

    size_value=$(echo $1 | sed 's/[^0-9]//g')
    size_4mb=$((size_value * 1024 / 4))
    size_hex=$(printf "%08x" ${size_4mb})
    echo "config writebooster size to $1 GB"

    ufs-utils desc -t 1 -r -D /desc_config -p /dev/bsg/ufs-bsg0 >/dev/null
    echo -n 01 | xxd -r -p | dd conv=notrunc of=/desc_config bs=1 seek=$((0x10)) count=1
    echo -n 01 | xxd -r -p | dd conv=notrunc of=/desc_config bs=1 seek=$((0x11)) count=1
    echo -n ${size_hex} | xxd -r -p | dd conv=notrunc of=/desc_config bs=1 seek=$((0x12)) count=4

    #config to ufs device
    sleep 1
    ufs-utils desc -t 1 -w /desc_config -p /dev/bsg/ufs-bsg0

    #get result
    ufs-utils desc -t 1 -r -D /desc_config -p /dev/bsg/ufs-bsg0

    echo "config writebooster done"
}

function ufs_lun_config() {
    echo "config lun start..."

    #get Configuration Descriptor
    ufs-utils desc -t 1 -r -D /desc_config -p /dev/bsg/ufs-bsg0 >/dev/null

    #lun0
    max_alloc_unit=$(ufs-utils desc -t 7 -r -p /dev/bsg/ufs-bsg0 | grep dEnhanced1MaxNAllocU | awk '{print $NF}')
    if [ -z "${max_alloc_unit}" ]; then
        echo "get max_alloc_unit err"
        exit 1
    fi
    lun0_num_alloc_unit_hex=$(printf "%08x" $(($max_alloc_unit - 0x30)))
    ##bLUEnable			00h
    echo -n 01 | xxd -r -p | dd conv=notrunc of=/desc_config bs=1 seek=$((0x16 + 0x0)) count=1
    ##dNumAllocUnits	04h
    echo -n ${lun0_num_alloc_unit_hex} | xxd -r -p | dd conv=notrunc of=/desc_config bs=1 seek=$((0x16 + 0x4)) count=4
    ##bProvisioningType	0Ah
    echo -n 02 | xxd -r -p | dd conv=notrunc of=/desc_config bs=1 seek=$((0x16 + 0xa)) count=1

    #bootlun enable
    ##bBootEnable
    echo -n 01 | xxd -r -p | dd conv=notrunc of=/desc_config bs=1 seek=$((0x3)) count=1

    #lun1 32MB boot partition, slc
    ##bLUEnable 		00h
    echo -n 01 | xxd -r -p | dd conv=notrunc of=/desc_config bs=1 seek=$((0x16 + 0x1a + 0x0)) count=1
    ##bBootLunID		01h
    echo -n 01 | xxd -r -p | dd conv=notrunc of=/desc_config bs=1 seek=$((0x16 + 0x1a + 0x1)) count=1
    ##bMemoryType		03h
    echo -n 03 | xxd -r -p | dd conv=notrunc of=/desc_config bs=1 seek=$((0x16 + 0x1a + 0x3)) count=1
    ##dNumAllocUnits	04h
    echo -n 00000018 | xxd -r -p | dd conv=notrunc of=/desc_config bs=1 seek=$((0x16 + 0x1a + 0x4)) count=4
    ##bDataReliability	08h
    echo -n 01 | xxd -r -p | dd conv=notrunc of=/desc_config bs=1 seek=$((0x16 + 0x1a + 0x8)) count=1
    ##bProvisioningType	0Ah
    echo -n 02 | xxd -r -p | dd conv=notrunc of=/desc_config bs=1 seek=$((0x16 + 0x1a + 0xa)) count=1

    #lun2 32MB boot partition, slc
    ##bLUEnable 		00h
    echo -n 01 | xxd -r -p | dd conv=notrunc of=/desc_config bs=1 seek=$((0x16 + 0x1a + 0x1a + 0x0)) count=1
    ##bBootLunID		01h
    echo -n 02 | xxd -r -p | dd conv=notrunc of=/desc_config bs=1 seek=$((0x16 + 0x1a + 0x1a + 0x1)) count=1
    ##bMemoryType		03h
    echo -n 03 | xxd -r -p | dd conv=notrunc of=/desc_config bs=1 seek=$((0x16 + 0x1a + 0x1a + 0x3)) count=1
    ##dNumAllocUnits	04h
    echo -n 00000018 | xxd -r -p | dd conv=notrunc of=/desc_config bs=1 seek=$((0x16 + 0x1a + 0x1a + 0x4)) count=4
    ##bDataReliability	08h
    echo -n 01 | xxd -r -p | dd conv=notrunc of=/desc_config bs=1 seek=$((0x16 + 0x1a + 0x1a + 0x8)) count=1
    ##bProvisioningType	0Ah
    echo -n 02 | xxd -r -p | dd conv=notrunc of=/desc_config bs=1 seek=$((0x16 + 0x1a + 0x1a + 0xa)) count=1

    #config to ufs device
    sleep 1
    ufs-utils desc -t 1 -w /desc_config -p /dev/bsg/ufs-bsg0

    #get result
    ufs-utils desc -t 1 -r -D /desc_config -p /dev/bsg/ufs-bsg0

    echo "config lun done"
}

tool_check
if [ "$1" == "wb" ] && [ -n "$2" ]; then
    ufs_wb_config "$2"
elif [ "$1" == "lun" ]; then
    ufs_lun_config
else
    help
fi

echo -en "${RED}After the restart, need to re-write the partition table!!!${NC}\n\n"
exit 0
